# coding=utf-8
import json
import tornado.ioloop
import tornado.web
from tornado.web import RequestHandler, Application
from adslproxy.db import RedisClient
from adslproxy.settings import *
from loguru import logger


class Server(RequestHandler):
    """
    服务器，对接 Redis 并提供 API
    """
    
    def initialize(self, redis):
        """
        初始化
        :param redis:
        :return:
        """
        self.redis = redis
    
    def get(self, api=''):
        """
        API 列表
        :param api:
        :return:
        """
        if not api:
            links = ['random', 'proxies', 'names', 'all', 'count']
            self.write('<h4>Welcome to ADSL Proxy API</h4>')
            for link in links:
                self.write('<a href=' + link + '>' + link + '</a><br>')
        
        if api == 'random':
            location = self.get_query_argument('location', None)
            result = self.redis.random(location)
            if result:
                self.write(result)
        
        if api == 'names':
            result = self.redis.names()
            if result:
                self.write(json.dumps(result))
        
        if api == 'proxies':
            result = self.redis.proxies()
            if result:
                self.write(json.dumps(result))
        
        if api == 'all':
            result = self.redis.all()
            if result:
                self.write(json.dumps(result))
        
        if api == 'count':
            self.write(str(self.redis.count()))


def serve(redis=None, port=SERVER_PORT, address=SERVER_HOST):
    if not redis:
        redis = RedisClient()
    application = Application([
        (r'/', Server, dict(redis=redis)),
        (r'/(.*)', Server, dict(redis=redis)),
    ])
    application.listen(port, address=address)
    logger.info(f'API listening on http://{address}:{port}')
    tornado.ioloop.IOLoop.instance().start()


if __name__ == '__main__':
    serve()



import time
from requests import ReadTimeout
from adslproxy.db import RedisClient
import requests
from adslproxy import settings
from collections import defaultdict
from loguru import logger


class Checker(object):
    
    def __init__(self):
        self.db = RedisClient()
        self.counts = defaultdict(int)
    
    def check(self, proxy):
        """
        测试代理，返回测试结果
        :param proxy: 代理
        :return: 测试结果
        """
        try:
            response = requests.get(settings.TEST_URL, proxies={
                'http': 'http://' + proxy,
                'https': 'https://' + proxy
            }, timeout=settings.TEST_TIMEOUT)
            logger.debug(f'Using {proxy} to test {settings.TEST_URL}...')
            if response.status_code == 200:
                return True
        except (ConnectionError, ReadTimeout):
            return False
    
    def run(self):
        """
        测试一轮
        :return:
        """
        proxies = self.db.all()
        logger.info(f'Try to get all proxies {proxies}')
        for name, proxy in proxies.items():
            # 检测无效
            if not self.check(proxy):
                logger.info(f'Proxy {proxy} invalid')
                self.counts[proxy] += 1
            else:
                logger.info(f'Proxy {proxy} valid')
            count = self.counts.get(proxy) or 0
            logger.debug(f'Count {count}, TEST_MAX_ERROR_COUNT {settings.TEST_MAX_ERROR_COUNT}')
            if count >= settings.TEST_MAX_ERROR_COUNT:
                self.db.remove(name)
    
    def loop(self):
        """
        循环测试
        :return:
        """
        while True:
            logger.info('Check for infinite')
            self.run()
            logger.info(f'Tested, sleeping for {settings.TEST_CYCLE}s...')
            time.sleep(settings.TEST_CYCLE)


def check(loop=True):
    """
    check proxies
    :param loop:
    :return:
    """
    if not loop:
        settings.TEST_MAX_ERROR_COUNT = 1
    checker = Checker()
    checker.loop() if loop else checker.run()


if __name__ == '__main__':
    check()



# coding=utf-8
from environs import Env

env = Env()

# 拨号间隔，单位秒
DIAL_CYCLE = env.int('DIAL_CYCLE', 100)
# 拨号出错重试间隔
DIAL_ERROR_CYCLE = env.int('DIAL_ERROR_CYCLE', 5)
# 拨号命令
DIAL_BASH = env.str('DIAL_BASH', 'adsl-stop;adsl-start')
# 拨号网卡
DIAL_IFNAME = env.str('DIAL_IFNAME', 'ppp0')

# 客户端唯一标识
CLIENT_NAME = env.str('CLIENT_NAME', 'adsl1')

# Redis数据库IP
REDIS_HOST = env.str('REDIS_HOST', '124.220.222.112')
# Redis数据库密码, 如无则填None
REDIS_PASSWORD = env.str('REDIS_PASSWORD', '1111')
# Redis数据库端口
REDIS_PORT = env.int('REDIS_PORT', 33479)
# 代理池键名
REDIS_KEY = env.str('REDIS_KEY', 'adsl')

# 测试URL
TEST_URL = env.str('TEST_URL', 'http://www.baidu.com')
# 测试最大失败次数
TEST_MAX_ERROR_COUNT = env.int('TEST_MAX_ERROR_COUNT', 10)
# 测试超时时间
TEST_TIMEOUT = env.int('TEST_TIMEOUT', 20)
# 测试周期
TEST_CYCLE = env.int('TEST_CYCLE', 100)

# 服务器端口
SERVER_PORT = env.int('SERVER_PORT', 8425)
SERVER_HOST = env.str('SERVER_HOST', '0.0.0.0')

# 代理端口
PROXY_PORT = env.int('PROXY_PORT', 3128)
PROXY_USERNAME = env.str('PROXY_USERNAME', '')
PROXY_PASSWORD = env.str('PROXY_PASSWORD', '')
API_URL = env.str('API_URL', '')
API_SIGNATURE = env.str('API_SIGNATURE', '')



# coding=utf-8
import redis
import random
from adslproxy.settings import *


class RedisClient(object):
    def __init__(self, host=REDIS_HOST, port=REDIS_PORT, password=REDIS_PASSWORD, redis_key=REDIS_KEY):
        """
        初始化Redis连接
        :param host: Redis 地址
        :param port: Redis 端口
        :param password: Redis 密码
        :param redis_key: Redis 哈希表名
        """
        self.db = redis.StrictRedis(host=host, port=port, password=password, decode_responses=True)
        self.redis_key = redis_key
    
    def set(self, name, proxy):
        """
        设置代理
        :param name: 主机名称
        :param proxy: 代理
        :return: 设置结果
        """
        return self.db.hset(self.redis_key, name, proxy)
    
    def get(self, name):
        """
        获取代理
        :param name: 主机名称
        :return: 代理
        """
        return self.db.hget(self.redis_key, name)
    
    def count(self):
        """
        获取代理总数
        :return: 代理总数
        """
        return self.db.hlen(self.redis_key)
    
    def remove(self, name):
        """
        删除代理
        :param name: 主机名称
        :return: 删除结果
        """
        return self.db.hdel(self.redis_key, name)
    
    def names(self):
        """
        获取主机名称列表
        :return: 获取主机名称列表
        """
        return self.db.hkeys(self.redis_key)
    
    def proxies(self):
        """
        获取代理列表
        :return: 代理列表
        """
        return self.db.hvals(self.redis_key)
    
    def random(self, location):
        """
        随机获取代理
        :return:
        """
        if location is not None:
            proxies = self.db.hgetall(self.redis_key)
            proxies = [v for k, v in proxies.items() if location in k]
        else:
            proxies = self.proxies()
        return random.choice(proxies)
    
    def all(self):
        """
        获取字典
        :return:
        """
        return self.db.hgetall(self.redis_key)
    
    def close(self):
        """
        关闭 Redis 连接
        :return:
        """
        del self.db



import re
import time
import requests
from requests.exceptions import ConnectionError, ReadTimeout
from adslproxy.settings import *
import platform
from loguru import logger
from retrying import retry, RetryError
import hashlib
from urllib.parse import urlencode

if platform.python_version().startswith('2.'):
    import commands as subprocess
elif platform.python_version().startswith('3.'):
    import subprocess
else:
    raise ValueError('python version must be 2 or 3')


def generate_url_signature(payload, key):
    payload['timestamp'] = int(time.time())
    qs = ''
    for i in sorted(payload.items()):
        qs += f'{i[0]}{i[1]}'
    qs += key
    print(qs)
    payload['signature'] = hashlib.md5(qs.encode('utf-8')).hexdigest()
    return urlencode(payload)


class Sender(object):
    """
    拨号并发送到 Redis
    """

    def extract_ip(self):
        """
        获取本机IP
        :param ifname: 网卡名称
        :return:
        """
        (status, output) = subprocess.getstatusoutput('ifconfig')
        if not status == 0: return
        pattern = re.compile(DIAL_IFNAME + '.*?inet.*?(\d+\.\d+\.\d+\.\d+).*?netmask', re.S)
        result = re.search(pattern, output)
        if result:
            # 返回拨号后的 IP 地址
            return result.group(1)

    def test_proxy(self, proxy):
        """
        测试代理，返回测试结果
        :param proxy: 代理
        :return: 测试结果
        """
        try:
            response = requests.get(TEST_URL, proxies={
                'http': 'http://' + proxy,
                'https': 'http://' + proxy
            }, timeout=TEST_TIMEOUT)
            if response.status_code == 200:
                return True
        except (ConnectionError, ReadTimeout):
            return False

    @retry(retry_on_result=lambda x: x is not True, stop_max_attempt_number=10)
    def remove_proxy(self):
        """
        移除代理
        :return: None
        """
        logger.info(f'Removing {CLIENT_NAME}...')
        try:
            payload = {
                'client_name': CLIENT_NAME
            }

            qs = generate_url_signature(payload)

            url = f"{API_URL}/remove?{qs}"
            res = requests.get(url)
            if res.ok:
                logger.info(f'Result is {res.text} Removed {CLIENT_NAME} successfully')
                logger.info(f'Removed {CLIENT_NAME} successfully')
                return True
            else:
                logger.info(f'Result is {res.text} Removed {CLIENT_NAME} failed')
                return False
        except Exception as e:
            logger.info(f'Remove {CLIENT_NAME} failed')

    def set_proxy(self, proxy):
        """
        设置代理
        :param proxy: 代理
        :return: None
        """
        payload = {
            'client_name': CLIENT_NAME,
            'client_ip': proxy
        }

        qs = generate_url_signature(payload)
        url = f"{API_URL}/update?{qs}"
        res = requests.get(url)
        if res.ok:
            logger.info(f'Result is {res.text} Set {CLIENT_NAME} successfully')
            return True
        else:
            logger.info(f'Result is {res.text} Removed {CLIENT_NAME} failed')
            return False

    def loop(self):
        """
        循环拨号
        :return:
        """
        while True:
            logger.info('Starting dial...')
            self.run()
            time.sleep(DIAL_CYCLE)

    def run(self):
        """
        拨号主进程
        :return: None
        """
        logger.info('Dial started, remove proxy')
        try:
            self.remove_proxy()
        except RetryError:
            logger.error('Retried for max times, continue')
        # 拨号
        (status, output) = subprocess.getstatusoutput(DIAL_BASH)
        if not status == 0:
            logger.error('Dial failed')
        # 获取拨号 IP
        ip = self.extract_ip()
        if ip:
            logger.info(f'Get new IP {ip}')
            if PROXY_USERNAME and PROXY_PASSWORD:
                proxy = '{username}:{password}@{ip}:{port}'.format(username=PROXY_USERNAME,
                                                                   password=PROXY_PASSWORD,
                                                                   ip=ip, port=PROXY_PORT)
            else:
                proxy = '{ip}:{port}'.format(ip=ip, port=PROXY_PORT)
            time.sleep(10)
            if self.test_proxy(proxy):
                logger.info(f'Valid proxy {proxy}')
                # 将代理放入数据库
                self.set_proxy(proxy)
                time.sleep(DIAL_CYCLE)
            else:
                logger.error(f'Proxy invalid {proxy}')
        else:
            # 获取 IP 失败，重新拨号
            logger.error('Get IP failed, re-dialing')
            self.run()


def send(loop=True):
    sender = Sender()
    sender.loop() if loop else sender.run()


if __name__ == '__main__':
    send()



import argparse
from adslproxy import version
import sys
from adslproxy.checker.checker import check
from adslproxy.sender.sender import send
from adslproxy.server.server import serve

optional_title = 'optional arguments'


def str2bool(v):
    """
    convert string to bool
    :param v:
    :return:
    """
    if isinstance(v, bool):
        return v
    if v.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    return True


class CapitalisedHelpFormatter(argparse.HelpFormatter):
    def __init__(self, prog):
        super(CapitalisedHelpFormatter, self).__init__(prog,
                                                       indent_increment=2,
                                                       max_help_position=30,
                                                       width=200)
        self._action_max_length = 20
    
    def add_usage(self, usage, actions, groups, prefix=None):
        if prefix is None:
            prefix = 'Usage: '
        return super(CapitalisedHelpFormatter, self).add_usage(
            usage, actions, groups, prefix)
    
    class _Section(object):
        
        def __init__(self, formatter, parent, heading=None):
            self.formatter = formatter
            self.parent = parent
            self.heading = heading
            self.items = []
        
        def format_help(self):
            # format the indented section
            if self.parent is not None:
                self.formatter._indent()
            join = self.formatter._join_parts
            item_help = join([func(*args) for func, args in self.items])
            if self.parent is not None:
                self.formatter._dedent()
            
            # return nothing if the section was empty
            if not item_help:  return ''
            
            # add the heading if the section was non-empty
            if self.heading is not argparse.SUPPRESS and self.heading is not None:
                current_indent = self.formatter._current_indent
                if self.heading == optional_title:
                    heading = '%*s\n%s:\n' % (current_indent, '', self.heading.title())
                else:
                    heading = '%*s%s:' % (current_indent, '', self.heading.title())
            else:
                heading = ''
            
            return join(['\n', heading, item_help])


parser = argparse.ArgumentParser(description='ADSLProxy %s - Easily to achieve ADSL Proxy Pool' % version(),
                                 formatter_class=CapitalisedHelpFormatter, add_help=False)

parser.add_argument('-v', '--version', action='version', version=version(), help='Get version of ADSLProxy')
parser.add_argument('-h', '--help', action='help', help='Show this help message and exit')

subparsers = parser.add_subparsers(dest='command', title='Available commands', metavar='')

# serve
parser_serve = subparsers.add_parser('serve', help='Run Server')

# check
parser_check = subparsers.add_parser('check', help='Run Checker')
parser_check.add_argument('-l', '--loop', default=True, type=str2bool, nargs='?', help='Run checker for infinite')

# send
parser_send = subparsers.add_parser('send', help='Run Sender')
parser_send.add_argument('-l', '--loop', default=True, type=str2bool, nargs='?', help='Run sender for infinite')

# show help info when no args
if len(sys.argv[1:]) == 0:
    parser.print_help()
    parser.exit()


def cmd():
    """
    run from cmd
    :return:
    """
    args = parser.parse_args()
    command = args.command
    # run server
    if command == 'serve':
        serve()
    # dial and send proxy to redis
    elif command == 'send':
        send(args.loop)
    # check proxies in redis
    elif command == 'check':
        check(args.loop)


if __name__ == '__main__':
    cmd()
